package com.zanxiang.sdk.common.util;

import com.zanxiang.common.constant.Constants;
import com.zanxiang.sdk.domain.bo.WxPayConfigBO;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import java.nio.charset.StandardCharsets;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author xufeng
 * @date 2022/6/8 15:37
 */
public class WxPayUtil {

    private WxPayConfigBO wxPay;

    public WxPayUtil(WxPayConfigBO wxPay) {
        this.wxPay = wxPay;
    }

    /**
     * 基础参数
     *
     * @param packageParams
     */
    public void commonParams(SortedMap<Object, Object> packageParams) {
        String appId = wxPay.getAppId();
        String mch_id = wxPay.getMchId();
        String currTime = PayCommonUtil.getCurrTime();
        String strTime = currTime.substring(8);
        String strRandom = PayCommonUtil.buildRandom(4) + "";
        String nonce_str = strTime + strRandom;
        packageParams.put("appid", appId);
        packageParams.put("mch_id", mch_id);
        packageParams.put("nonce_str", nonce_str);
    }

    /**
     * 该接口主要用于扫码原生支付模式一中的二维码链接转成短链接(weixin://wxpay/s/XXXXXX)
     * 减小二维码数据量，提升扫描速度和精确度
     *
     * @param urlCode
     * @return
     */
    public String shortUrl(String urlCode) {
        try {
            String key = wxPay.getApiKey();
            SortedMap<Object, Object> packageParams = new TreeMap<>();
            commonParams(packageParams);
            packageParams.put("long_url", urlCode);
            String sign = PayCommonUtil.createSign("UTF-8", packageParams, key);
            packageParams.put("sign", sign);
            String requestXml = PayCommonUtil.getRequestXml(packageParams);
            String resXml = HttpUtil.postData(WxPayUrl.SHORT_URL, requestXml);
            Map map = XMLUtil.doXmlParse(resXml);
            String returnCode = (String) map.get("return_code");
            if (Constants.SUCCESS.equalsIgnoreCase(returnCode)) {
                String resultCode = (String) map.get("return_code");
                if (Constants.SUCCESS.equalsIgnoreCase(resultCode)) {
                    urlCode = (String) map.get("short_url");
                    return urlCode;
                } else {
                    return Constants.FAIL;
                }
            } else {
                return Constants.FAIL;
            }
        } catch (Exception e) {
            e.printStackTrace();
            return Constants.FAIL;
        }
    }

    /**
     * 转换deepLink
     *
     * @param url
     * @return
     * @throws Exception
     */
    public String readUrl(String url, String serverUrl) throws Exception {
        // 定义好匹配规则
        String regex = "weixin://wap/pay?.[^\\x{4e00}-\\x{9fa5}-\"]+";
        Pattern pattern = Pattern.compile(regex);
        URL payUrl = new URL(url);
        URLConnection con = payUrl.openConnection();
        // 加referer防盗链,必须是申请的支付白名单域名
        con.setRequestProperty("Referer", serverUrl);
        BufferedReader br = new BufferedReader(new InputStreamReader(con.getInputStream(), StandardCharsets.UTF_8));
        String result = null;
        String realLineStr;
        while ((realLineStr = br.readLine()) != null) {
            // 一行行查找
            Matcher matcher = pattern.matcher(realLineStr);
            if (matcher.find()) {
                result = matcher.group();
                break;
            }
        }
        return result;
    }

    public static String subZeroAndDot(String s) {
        if (s.indexOf(".") > 0) {
            // 去掉多余的0
            s = s.replaceAll("0+?$", "");
            // 如最后一位是.则去掉
            s = s.replaceAll("[.]$", "");
        }
        return s;
    }
}