package com.zanxiang.sdk.service.Impl.pay;

import com.alibaba.fastjson.JSONObject;
import com.zanxiang.common.constant.Constants;
import com.zanxiang.common.domain.ResultMap;
import com.zanxiang.common.enums.ResEnum;
import com.zanxiang.common.utils.JsonUtil;
import com.zanxiang.common.utils.RandomStringUtil;
import com.zanxiang.sdk.common.util.*;
import com.zanxiang.sdk.domain.bo.ProductPayAttachParamBO;
import com.zanxiang.sdk.domain.bo.ProductPayParamBO;
import com.zanxiang.sdk.domain.bo.WxPayConfigBO;
import com.zanxiang.sdk.service.OrderPayService;
import lombok.extern.slf4j.Slf4j;
import org.jdom.JDOMException;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import weixin.popular.api.SnsAPI;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.nio.charset.StandardCharsets;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

/**
 * @author xufeng
 * @date 2022/6/8 15:37
 */
@Component
@Slf4j
@Service("WxPayService")
public class WxPayServiceImpl extends PayService implements OrderPayService {

    /**
     * 二维码支付
     */
    private static final String WX_PAY_NATIVE = "NATIVE";

    /**
     * h5支付
     */
    private static final String WX_PAY_MWEB = "MWEB";

    /**
     * 服务器域名
     */
    @Value("${server.domain}")
    private String serverUrl;

    /**
     * 微信回调接口
     */
    @Value("${payConfig.wx.notifyUrl}")
    private String notifyUrl;

    /**
     * 微信支付配置
     */
    private WxPayConfigBO config;

    private WxPayUtil wxPayUtil;

    private String body;

    private String totalFee;

    private String outTradeNo;

    /**
     * 支付调起
     *
     * @param product
     * @return
     */
    @Override
    public ResultMap create(ProductPayParamBO product) {
        this.body = product.getSubject();
        this.totalFee = String.valueOf(Float.parseFloat(product.getTotalFee()) * 100);
        this.outTradeNo = product.getOutTradeNo() + RandomStringUtil.randomNumStr(5);
        this.attach = product.getAttach();
        //初始化配置
        this.configInit(getPayConfig(product.getGameId(), product.getPayWay(), product.getPayDevice()));
        switch (product.getPayDevice()) {
            case 1:
                //PC端二维码
                return this.mobile(product, WX_PAY_NATIVE);
            case 2:
                //手机端H5
                return this.mobile(product, WX_PAY_MWEB);
//            case 3:
//                return this.mobile(product, "MWEB");
            case 4:
                //微信小程序
                return this.mp(product);
            default:
                throw new RuntimeException("未知支付方式");
        }
    }

    /**
     * 异步回调
     *
     * @param request
     * @param response
     * @return
     */
    @Override
    public String notify(HttpServletRequest request, HttpServletResponse response) throws IOException, JDOMException {
        // 读取参数
        InputStream inputStream = request.getInputStream();
        StringBuffer sb = new StringBuffer();
        String s;
        BufferedReader in = new BufferedReader(new InputStreamReader(inputStream, StandardCharsets.UTF_8));
        while ((s = in.readLine()) != null) {
            sb.append(s);
        }
        in.close();
        inputStream.close();
        log.info("微信支付回调 body:{}", sb);
        // 解析xml成map
        Map m = XMLUtil.doXmlParse(sb.toString());
        // 过滤空 设置 TreeMap
        SortedMap<Object, Object> packageParams = new TreeMap<>();
        for (Object parameter : m.keySet()) {
            String parameterValue = (String) m.get(parameter);
            String v = "";
            if (null != parameterValue) {
                v = parameterValue.trim();
            }
            packageParams.put(parameter, v);
        }
        ProductPayAttachParamBO attachBO = JsonUtil.toObj(JsonUtil.toString(packageParams.get("attach")), ProductPayAttachParamBO.class);
        configInit(getPayConfig(attachBO.getGamePayWayId()));
        // 账号信息
        String key = config.getApiKey();
        // 判断签名是否正确
        if (PayCommonUtil.isTenPaySign(Constants.UTF8, packageParams, key)) {
            log.info("微信支付成功回调");
            // 处理业务开始
            String resXml;
            if (Constants.SUCCESS.equalsIgnoreCase(packageParams.get("result_code").toString())) {
                // 这里是支付成功
                String orderNo = (String) packageParams.get("out_trade_no");
                log.info("微信订单号{}付款成功", orderNo);
                if (paySuccess(attachBO.getOrderId(), String.valueOf(Float.parseFloat(packageParams.get("total_amount").toString()) / 100), packageParams.get("trade_no").toString(), attachBO.getGamePayWayId())) {
                    log.info("微信异步回调成功 request:{},商户订单号为：{}", request, outTradeNo);
                    return ResEnum.SUCCESS.getMsg();
                }
                // 通知微信.异步确认成功.必写.不然会一直通知后台.八次之后就认为交易失败了
                resXml = "<xml>" + "<return_code><![CDATA[SUCCESS]]></return_code>" + "<return_msg><![CDATA[OK]]></return_msg>" + "</xml> ";
            } else {
                log.info("支付失败,错误信息：{}", packageParams.get("err_code"));
                resXml = "<xml>" + "<return_code><![CDATA[FAIL]]></return_code>" + "<return_msg><![CDATA[报文为空]]></return_msg>" + "</xml> ";
            }
            // 处理业务完毕
            BufferedOutputStream out = new BufferedOutputStream(response.getOutputStream());
            out.write(resXml.getBytes());
            out.flush();
            out.close();
        } else {
            log.info("通知签名验证失败");
        }
        return null;
    }

    /**
     * 同步回调
     *
     * @param request
     * @return
     */

    @Override
    public ResultMap synNotify(HttpServletRequest request) {
        log.info("微信支付无同步回调 request:{}", request);
        return null;
    }


    /**
     * 小程序(需要HTTPS)
     *
     * @param product
     * @return
     */
    private ResultMap mp(ProductPayParamBO product) {
        //redirect_uri 需要在微信支付端添加认证网址
        totalFee = WxPayUtil.subZeroAndDot(this.totalFee);
        String redirectUri = serverUrl + "/api/sdk/weixinMobile/dopay?outTradeNo=" + product.getOutTradeNo() + "&totalFee=" + totalFee;
        //也可以通过state传递参数 redirect_uri 后面加参数未经过验证
        String result = SnsAPI.connectOauth2Authorize(config.getAppId(), redirectUri, true, null);
        return ResultMap.ok(result);
    }

    /**
     * H5手机支付（jspapi）
     *
     * @param product
     * @return
     */
    private ResultMap mobile(ProductPayParamBO product, String tradeType) {
        //商户支付回调URL设置指引：进入公众平台-->微信支付-->开发配置-->扫码支付-->修改 加入回调URL
        log.info("订单号：{}生成微信支付码", product.getOutTradeNo());
        try {
            // 账号信息
            String key = config.getApiKey();
            SortedMap<Object, Object> packageParams = new TreeMap<>();
            wxPayUtil.commonParams(packageParams);
            // 商品描述
            packageParams.put("body", this.body);
            // 商户订单号
            packageParams.put("out_trade_no", this.outTradeNo);
            totalFee = WxPayUtil.subZeroAndDot(this.totalFee);
            // 总金额
            packageParams.put("total_fee", totalFee);
            // 发起人IP地址
            packageParams.put("spbill_create_ip", product.getSpbillCreateIp());
            // 回调地址 notifyUrl
            packageParams.put("notify_url", notifyUrl);
            // 交易类型
            packageParams.put("trade_type", tradeType);
            packageParams.put("attach", JSONObject.toJSONString(this.attach));
            String sign = PayCommonUtil.createSign(Constants.UTF8, packageParams, key);
            packageParams.put("sign", sign);
            String requestXml = PayCommonUtil.getRequestXml(packageParams);
            String resXml = HttpUtil.postData(WxPayUrl.UNIFIED_ORDER_URL, requestXml);
            Map map = XMLUtil.doXmlParse(resXml);
            if (map == null) {
                log.error("微信支付通信异常, 订单号：{}", product.getOutTradeNo());
                return ResultMap.error();
            }
            String returnCode = (String) map.get("return_code");
            //返回失败
            if (!Constants.SUCCESS.equalsIgnoreCase(returnCode)) {
                String returnMsg = (String) map.get("return_msg");
                log.error("微信支付通信失败, 订单号：{}, msg :{}", product.getOutTradeNo(), returnMsg);
                return ResultMap.error(returnMsg);
            }
            //结果失败
            String resultCode = (String) map.get("result_code");
            if (!Constants.SUCCESS.equalsIgnoreCase(resultCode)) {
                String errCodeDes = (String) map.get("err_code_des");
                log.error("微信支付通信失败, 订单号：{}, errCodeDes :{}", product.getOutTradeNo(), errCodeDes);
                return ResultMap.error(errCodeDes);
            }
            if (tradeType.equals(WX_PAY_MWEB)) {
                String urlCode = wxPayUtil.readUrl((String) map.get("mweb_url"), serverUrl);
                return ResultMap.ok(product.getPayDevice(), urlCode);
            }
            if (tradeType.equals(WX_PAY_NATIVE)) {
                String urlCode = (String) map.get("code_url");
                return ResultMap.ok(product.getPayDevice(), urlCode);
            }
            return ResultMap.ok(product.getPayDevice(), null);
        } catch (Exception e) {
            log.error("微信支付通信异常, 订单号：{}, e : {}", product.getOutTradeNo(), e.getMessage());
            return ResultMap.error();
        }
    }

    /**
     * 配置初始化
     */
    @Override
    public void configInit(String obj) {
        this.config = JSONObject.parseObject(obj, WxPayConfigBO.class);
        this.wxPayUtil = new WxPayUtil(this.config);
    }
}
